{
    "aggregate": {
        "availability_zone": "nova2",
        "created_at": "%(strtime)s",
        "deleted": false,
        "deleted_at": null,
        "hosts": [],
        "id": 1,
        "metadata": {
            "availability_zone": "nova2"
        },
        "name": "newname",
        "updated_at": "%(strtime)s"
    }
}
