{
    "quota_set": {
        "cores": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "fixed_ips": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "floating_ips": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "id": "fake_tenant",
        "injected_file_content_bytes": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "injected_file_path_bytes": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "injected_files": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "instances": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "key_pairs": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "metadata_items": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "ram": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "security_group_rules": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "security_groups": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "server_group_members": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        },
        "server_groups": {
            "in_use": -1,
            "limit": -1,
            "reserved": -1
        }
    }
}
