# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# TODO(stephenfin): Restrict the value to 'null' in a future API version
suspend = {
    'type': 'object',
    'properties': {
        'suspend': {},
    },
    'required': ['suspend'],
    'additionalProperties': False
}

# TODO(stephenfin): Restrict the value to 'null' in a future API version
resume = {
    'type': 'object',
    'properties': {
        'resume': {},
    },
    'required': ['resume'],
    'additionalProperties': False
}

suspend_response = {
    'type': 'null',
}

resume_response = {
    'type': 'null',
}
